@@ -37,6 +37,7 @@
 /* forward declaration for internal use */
 static void HandleTapeButtons(struct GadgetInfo *);
 static void TapeStopWarpForward();
+static float GetTapeLengthSecondsFloat();
 
 static struct GadgetInfo *tape_gadget[NUM_TAPE_BUTTONS];
 
@@ -484,11 +485,13 @@ void PrintTapeReplayProgress(boolean replay_finished)
   }
   else
   {
+    float tape_length_seconds = GetTapeLengthSecondsFloat();
+
     PrintNoLog("\r");
     Print("Level %03d [%02d:%02d]: (%02d:%02d.%03d / %.2f %%) - %s.\n",
 	  level_nr, tape.length_seconds / 60, tape.length_seconds % 60,
 	  counter_seconds / 60, counter_seconds % 60, counter % 1000,
-	  (float)counter / tape.length_seconds / 10,
+	  (float)counter / tape_length_seconds / 10,
 	  tape.auto_play_level_solved ? "solved" : "NOT SOLVED");
 
     counter_last = -1;
@@ -637,11 +640,11 @@ static void TapeAppendRecording()
 
 void TapeHaltRecording()
 {
-  if (!tape.recording)
-    return;
-
   tape.counter++;
-  tape.pos[tape.counter].delay = 0;
+
+  // initialize delay for next tape entry (to be able to continue recording)
+  if (tape.counter < MAX_TAPE_LEN)
+    tape.pos[tape.counter].delay = 0;
 
   tape.length = tape.counter;
   tape.length_frames = GetTapeLengthFrames();
@@ -650,7 +653,8 @@ void TapeHaltRecording()
 
 void TapeStopRecording()
 {
-  TapeHaltRecording();
+  if (tape.recording)
+    TapeHaltRecording();
 
   tape.recording = FALSE;
   tape.pausing = FALSE;
@@ -659,33 +663,10 @@ void TapeStopRecording()
   MapTapeEjectButton();
 }
 
-void TapeRecordAction(byte action_raw[MAX_PLAYERS])
+boolean TapeAddAction(byte action[MAX_PLAYERS])
 {
-  byte action[MAX_PLAYERS];
   int i;
 
-  if (!tape.recording)		/* (record action even when tape is paused) */
-    return;
-
-  if (tape.counter >= MAX_TAPE_LEN - 1)
-  {
-    TapeStopRecording();
-    return;
-  }
-
-  for (i = 0; i < MAX_PLAYERS; i++)
-    action[i] = action_raw[i];
-
-  if (tape.set_centered_player)
-  {
-    for (i = 0; i < MAX_PLAYERS; i++)
-      if (tape.centered_player_nr_next == i ||
-	  tape.centered_player_nr_next == -1)
-	action[i] |= KEY_SET_FOCUS;
-
-    tape.set_centered_player = FALSE;
-  }
-
   if (tape.pos[tape.counter].delay > 0)		/* already stored action */
   {
     boolean changed_events = FALSE;
@@ -696,6 +677,9 @@ void TapeRecordAction(byte action_raw[MAX_PLAYERS])
 
     if (changed_events || tape.pos[tape.counter].delay >= 255)
     {
+      if (tape.counter >= MAX_TAPE_LEN - 1)
+	return FALSE;
+
       tape.counter++;
       tape.pos[tape.counter].delay = 0;
     }
@@ -710,6 +694,33 @@ void TapeRecordAction(byte action_raw[MAX_PLAYERS])
 
     tape.pos[tape.counter].delay++;
   }
+
+  return TRUE;
+}
+
+void TapeRecordAction(byte action_raw[MAX_PLAYERS])
+{
+  byte action[MAX_PLAYERS];
+  int i;
+
+  if (!tape.recording)		/* (record action even when tape is paused) */
+    return;
+
+  for (i = 0; i < MAX_PLAYERS; i++)
+    action[i] = action_raw[i];
+
+  if (tape.set_centered_player)
+  {
+    for (i = 0; i < MAX_PLAYERS; i++)
+      if (tape.centered_player_nr_next == i ||
+	  tape.centered_player_nr_next == -1)
+	action[i] |= KEY_SET_FOCUS;
+
+    tape.set_centered_player = FALSE;
+  }
+
+  if (!TapeAddAction(action))
+    TapeStopRecording();
 }
 
 void TapeTogglePause(boolean toggle_mode)
@@ -935,6 +946,11 @@ unsigned int GetTapeLengthSeconds()
   return (GetTapeLengthFrames() * GAME_FRAME_DELAY / 1000);
 }
 
+static float GetTapeLengthSecondsFloat()
+{
+  return ((float)GetTapeLengthFrames() * GAME_FRAME_DELAY / 1000);
+}
+
 static void TapeStartWarpForward(int mode)
 {
   tape.fast_forward = (mode & AUTOPLAY_FFWD);
@@ -1066,13 +1082,23 @@ void TapeQuickLoad()
 
 void InsertSolutionTape()
 {
-  if (!TAPE_IS_EMPTY(tape))
+  boolean level_has_tape = (level.game_engine_type == GAME_ENGINE_TYPE_SP &&
+			    level.native_sp_level->demo.is_available);
+
+  if (!fileExists(getSolutionTapeFilename(level_nr)) && !level_has_tape)
+  {
+    Request("No solution tape for this level!", REQ_CONFIRM);
+
     return;
+  }
+
+  // if tape recorder already contains a tape, remove it without asking
+  TapeErase();
 
   LoadSolutionTape(level_nr);
 
   if (TAPE_IS_EMPTY(tape))
-    Request("No solution tape for this level!", REQ_CONFIRM);
+    Request("Loading solution tape for this level failed!", REQ_CONFIRM);
 
   DrawCompleteVideoDisplay();
 }
@@ -1384,7 +1410,7 @@ static void HandleTapeButtonsExt(int id)
       else
       {
 	if (tape.changed)
-	  SaveTapeChecked(tape.level_nr);
+	  SaveTapeChecked(level_nr);
 
 	TapeErase();
       }
