@@ -1,15 +1,13 @@
-/***********************************************************
-* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
-*----------------------------------------------------------*
-* (c) 1995-2006 Artsoft Entertainment                      *
-*               Holger Schemel                             *
-*               Detmolder Strasse 189                      *
-*               33604 Bielefeld                            *
-*               Germany                                    *
-*               e-mail: info@artsoft.org                   *
-*----------------------------------------------------------*
-* tools.h                                                  *
-***********************************************************/
+// ============================================================================
+// Rocks'n'Diamonds - McDuffin Strikes Back!
+// ----------------------------------------------------------------------------
+// (c) 1995-2014 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    http://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// tools.h
+// ============================================================================
 
 #ifndef TOOLS_H
 #define TOOLS_H
@@ -17,9 +15,6 @@
 #include "main.h"
 #include "game.h"
 
-/* for SetDrawtoField */
-#define DRAW_BUFFERED		1
-#define DRAW_BACKBUFFER		2
 
 /* for DrawElementShifted */
 #define NO_CUTTING		0
@@ -49,6 +44,16 @@
 #define DOOR_GET_STATE		(1 << 8)
 #define DOOR_SET_STATE		(1 << 9)
 
+#define DOOR_1			(DOOR_ACTION_1)
+#define DOOR_2			(DOOR_ACTION_2)
+#define DOOR_OPEN		(DOOR_OPEN_ALL)
+#define DOOR_CLOSE		(DOOR_CLOSE_ALL)
+
+#define DOOR_INDEX_FROM_TOKEN(x)	((x) == DOOR_1 ? 0 : 1)
+#define DOOR_TOKEN_FROM_INDEX(x)	((x) == 0 ? DOOR_1 ? : DOOR_2)
+#define REDRAW_DOOR_FROM_TOKEN(x)	((x) == DOOR_1 ? REDRAW_DOOR_1 : \
+					 REDRAW_DOOR_2)
+
 /* for Request */
 #define REQ_ASK			(1 << 0)
 #define REQ_CONFIRM		(1 << 1)
@@ -59,7 +64,15 @@
 
 #define REQUEST_WAIT_FOR_INPUT	(REQ_ASK | REQ_CONFIRM | REQ_PLAYER)
 
+
+int correctLevelPosX_EM(int);
+int correctLevelPosY_EM(int);
+
+int getLevelFromScreenX(int);
+int getLevelFromScreenY(int);
+
 void DumpTile(int, int);
+void DumpTileFromScreen(int, int);
 
 void DrawMaskedBorder_FIELD();
 void DrawMaskedBorder_DOOR_1();
@@ -67,11 +80,14 @@ void DrawMaskedBorder_DOOR_2();
 void DrawMaskedBorder_DOOR_3();
 void DrawMaskedBorder_ALL();
 void DrawMaskedBorder(int);
+void DrawMaskedBorderToTarget(int);
 
 void SetDrawtoField(int);
-void RedrawPlayfield(boolean, int, int, int, int);
+void RedrawPlayfield();
+void BlitScreenToBitmap_RND(Bitmap *);
 void BlitScreenToBitmap(Bitmap *);
 void BackToFront();
+void BackToFront_WithFrameDelay(unsigned int);
 
 void FadeIn(int);
 void FadeOut(int);
@@ -85,6 +101,8 @@ void FadeSetDisabled();
 void FadeSkipNextFadeIn();
 void FadeSkipNextFadeOut();
 
+Bitmap *getGlobalBorderBitmapFromStatus(int);
+
 void ClearField();
 void SetWindowBackgroundImageIfDefined(int);
 void SetMainBackgroundImageIfDefined(int);
@@ -96,6 +114,8 @@ void SetPanelBackground();
 void DrawBackground(int, int, int, int);
 void DrawBackgroundForFont(int, int, int, int, int);
 void DrawBackgroundForGraphic(int, int, int, int, int);
+boolean CheckIfGlobalBorderHasChanged();
+void RedrawGlobalBorder();
 
 void MarkTileDirty(int, int);
 void SetBorderElement();
@@ -105,9 +125,6 @@ void FloodFillLevel(int, int, int, short[MAX_LEV_FIELDX][MAX_LEV_FIELDY],
 void SetRandomAnimationValue(int, int);
 int getGraphicAnimationFrame(int, int);
 
-void DrawGraphicAnimation(int, int, int);
-void DrawGraphicAnimationExt(DrawBuffer *, int, int, int, int, int);
-
 void DrawFixedGraphicAnimation(int, int, int);
 void DrawFixedGraphicAnimationExt(DrawBuffer *, int, int, int, int, int);
 
@@ -120,12 +137,12 @@ void DrawAllPlayers(void);
 void DrawPlayerField(int, int);
 void DrawPlayer(struct PlayerInfo *);
 
+void getGraphicSourceBitmap(int, int, Bitmap **);
+void getGraphicSourceXY(int, int, int *, int *, boolean);
 void getSizedGraphicSourceExt(int, int, int, Bitmap **, int *, int *, boolean);
-void getFixedGraphicSourceExt(int, int, Bitmap **, int *, int *, boolean);
 void getSizedGraphicSource(int, int, int, Bitmap **, int *, int *);
 void getFixedGraphicSource(int, int, Bitmap **, int *, int *);
 void getMiniGraphicSource(int, Bitmap **, int *, int *);
-void getGraphicSourceExt(int, int, Bitmap **, int *, int *, boolean);
 void getGraphicSource(int, int, Bitmap **, int *, int *);
 
 void DrawGraphic(int, int, int, int);
@@ -159,18 +176,25 @@ void DrawLevelElement(int, int, int);
 void DrawScreenField(int, int);
 void DrawLevelField(int, int);
 
+void DrawSizedElement(int, int, int, int);
+void DrawSizedElementOrWall(int, int, int, int, int);
+
 void DrawMiniElement(int, int, int);
 void DrawMiniElementOrWall(int, int, int, int);
 
 void ShowEnvelope(int);
 void ShowEnvelopeDoor(char *text, int);
 
-void DrawLevel(void);
+void DrawLevel(int);
+void DrawSizedLevel(int, int, int, int, int);
 void DrawMiniLevel(int, int, int, int);
-void DrawPreviewLevel(boolean);
+void DrawPreviewLevelInitial(void);
+void DrawPreviewLevelAnimation(void);
 
-void WaitForEventToContinue();
+void WaitForEventToContinue(void);
 boolean Request(char *, unsigned int);
+void InitGraphicCompatibilityInfo_Doors(void);
+void InitDoors(void);
 unsigned int OpenDoor(unsigned int);
 unsigned int CloseDoor(unsigned int);
 unsigned int GetDoorState(void);
@@ -228,10 +252,17 @@ void PlayMenuSoundIfLoopExt(int);
 void PlayMenuSoundIfLoop();
 void PlayMenuMusicExt(int);
 void PlayMenuMusic();
+void PlayMenuSoundsAndMusic();
+void FadeMenuSoundsAndMusic();
 void PlaySoundActivating();
 void PlaySoundSelecting();
 
-void ToggleFullscreenIfNeeded();
+void SetAnimStatus(int);
+void SetGameStatus(int);
+void SetFontStatus(int);
+void ResetFontStatus();
+
+void ToggleFullscreenOrChangeWindowScalingIfNeeded();
 void ChangeViewportPropertiesIfNeeded();
 
 #endif	/* TOOLS_H */
